<?php

namespace Drupal\fpdi_print;

use Drupal\Component\Utility\Html;
use setasign\Fpdi\Tcpdf\Fpdi;

class Pdf extends Fpdi {

  public $headerHTML = '';
  public $footerHTML = '';
  public $footerHeight = 20;
  /**
   * Draw an imported PDF logo on every page
   */
  public function Header()
  {
    if(!empty($this->headerHTML)){
      $this->writeHTML($this->headerHTML, FALSE, 0, TRUE, 0);
    }
  }

  public function Footer()
  {
    if(!empty($this->footerHTML)) {
      $this->writeHTMLCell(null,$this->footerHeight, null ,-$this->footerHeight,$this->footerHTML);
    }
  }
}
/**
 * The print builder service.
 */
class PrintBuilder {

  /*
   * array x, y coordinate positions in the page, the page number starts with 1
   * @param array $positions
   *
   * pdf template file path or fid
   * @param string $filePdfTemplate
   *
   * define array header ['logo'|'width'|'title'|'string'|'text_color'|'line_color']
   * @param array $header
   *
   * define array footer ['text_color'|'line_color']
   * @param array $footer
   *
   * @return \setasign\Fpdi\Tcpdf\Fpdi
   */
  public function getPDF($positions = '', $filePdfTemplate = '', $header = FALSE, $footer = FALSE) {
    if(is_string($header) || is_string($footer)){
      $pdf = new Pdf();
      $pdf->headerHTML = $header;
      $pdf->footerHTML = $footer;
    }else{
      $pdf = new Fpdi();
      if($header){
        $logo = !empty($header['logo']) ? $header['logo'] : '';
        $logo_width = !empty($header['width']) ? $header['width'] : '';
        $title = !empty($header['title']) ? $header['title'] : '';
        $string = !empty($header['string']) ? $header['string'] : '';
        $text_color = !empty($header['text_color']) ? $this->hex2rgb($header['text_color']) : array(0,0,0);
        $line_color = !empty($header['line_color']) ? $this->hex2rgb($header['line_color']) : array(0,0,0);
        $pdf->setHeaderData($logo,$logo_width,$title,$string,$text_color,$line_color);
      }else{
        $pdf->setPrintHeader($header);
      }
      if($footer){
        $text_color = !empty($footer['text_color']) ? $this->hex2rgb($footer['text_color']) : array(0,0,0);
        $line_color = !empty($footer['line_color']) ? $this->hex2rgb($footer['line_color']) : array(0,0,0);
        $pdf->setFooterData($text_color,$line_color);
      }else{
        $pdf->setPrintFooter($footer);
      }
      $pdf->SetAutoPageBreak(FALSE, 0);
    }
    $currentUser = \Drupal::currentUser();
    if(!empty($currentUser)){
      $pdf->SetAuthor($currentUser->getDisplayName());
    }

    $pageCount = 1;
    $config = \Drupal::config('fpdi_print.settings');
    if(!empty($config->get('default_css'))){
      $html = '<style>'.file_get_contents($config->get('default_css')).'</style>';
      $pdf->writeHTML($html, true, false, true, false, '');
    }
    $hasTemplate = FALSE;

    //if file pdf template is fid
    if (!empty($filePdfTemplate) && is_numeric($filePdfTemplate)) {
      $filePdfTemplate = $this->getRealPathFromFid($filePdfTemplate);
    }
    if (!empty($filePdfTemplate) && file_exists($filePdfTemplate)) {
      $pageCount = $pdf->setSourceFile($filePdfTemplate);
      $hasTemplate = TRUE;
    }
    foreach (range(1, $pageCount) as $page) {
      if ($hasTemplate) {
        // import page 1
        $templateId = $pdf->importPage($page);
        // get the size of the imported page
        $size = $pdf->getTemplateSize($templateId);

        // create a page (landscape or portrait depending on the imported page size)
        if ($size[0] > $size[1]) {
          $pdf->AddPage('L', [$size[0], $size[1]]);
        }
        else {
          $pdf->AddPage('P', [$size[0], $size[1]]);
        }
        // use the imported page
        $pdf->useTemplate($templateId);
      }else{
        $pdf->AddPage($config->get('orientation'),$config->get('page_format'));
      }
      if (!empty($positions[$page])) {
        foreach ($positions[$page] as $position) {
          if (!empty(array_intersect(array_keys($position), ["text","image","html",]))) {
            if(empty($position['x'])) {
              $position['x'] = 0;
            }
            if(empty($position['y'])) {
              $position['y'] = 0;
            }
            if (!empty($position['text'])) {
              $position['text'] = $this->formatText($position['text']);
              $pdf->SetXY($position['x'], $position['y']);
              if (!empty($position['height'])) {
                $pdf->MultiCell(0, $position['height'], $position['text']);
              }
              else {
                $pdf->Write(0, $position['text']);
              }
            }
            if (!empty($position['image'])) {
              $position['image'] = $this->formatImage($position['image']);
              [$img_width, $img_height] = getimagesize($position['image']);
              $path_parts = pathinfo($position['image']);
              if (!empty($position['width'])) {
                $img_width = $position['width'];
                $img_height = $position['height'];
              }
              $pdf->Image($position['image'], $position['x'], $position['y'], $img_width, $img_height, strtolower($path_parts['extension']));
            }
            if (!empty($position['html'])) {
              $pdf->SetXY($position['x'], $position['y']);
              if (!empty($position['height'])) {
                $pdf->MultiCell(0, $position['height'], $pdf->WriteHTML($position['text']));
              }
              else {
                $pdf->writeHTML($position['html'], TRUE, 0, TRUE, 0);
              }
            }
          }
        }
      }

    }
    return $pdf;
  }

  /**
   * filter value
   */
  protected function formatText($value) {
    $value = Html::decodeEntities($value);
    $value = preg_replace('/<br(\s+\/)/', "\n", $value);
    $value = trim(strip_tags($value));
    return $value;
  }

  /**
   * get image path
   */
  protected function formatImage($img) {
    $img = Html::decodeEntities($img);
    if (is_numeric($img)) {
      $filename = $this->getRealPathFromFid($img);
    } else {
      $img = trim(strip_tags($img, '<img>'));
      $doc = new \DOMDocument();
      $doc->loadHTML($img);
      $xpath = new \DOMXPath($doc);
      $src = $xpath->evaluate("string(//img/@src)");
      if (!empty($src)) {
        $img = $src;
      }
      //remove url absolute
      $url = \Drupal\Core\Url::fromUserInput('/', ['absolute' => TRUE])
        ->toString();
      $img = str_replace($url, '', $img);
      $urlParse = parse_url($img);
      //Convert img to path
      $filename = \Drupal::root() . $urlParse['path'];
    }
    $is_picture = is_file($filename) && is_readable($filename) && getimagesize($filename);
    if ($is_picture) {
      return $filename;
    }
    return FALSE;
  }
  protected function hex2rgb ( $hex_color ) {
    $values = str_replace( '#', '', $hex_color );
    switch ( strlen( $values ) ) {
      case 3;
        [ $r, $g, $b ] = sscanf( $values, "%1s%1s%1s" );
        return [ hexdec( "$r$r" ), hexdec( "$g$g" ), hexdec( "$b$b" ) ];
      case 6;
        return array_map( 'hexdec', sscanf( $values, "%2s%2s%2s" ) );
      default:
        return false;
    }
  }
  protected function getRealPathFromFid(int $fid) {
    $file = \Drupal\file\Entity\File::load($fid);
    if(!empty($file)){
      $uri = $file->getFileUri();
      $stream_manager = \Drupal::service('stream_wrapper_manager')->getViaUri($uri);
      return $stream_manager->realpath();
    }
    return FALSE;
  }

}
